from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.workflow_execution_request import WorkflowExecutionRequest
from typing import cast
from typing import Dict
from uuid import UUID



def _get_kwargs(
    workflow_id: UUID,
    *,
    body: WorkflowExecutionRequest,

) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}






    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/workflow/{workflow_id}".format(workflow_id=workflow_id,),
    }

    _body = body.to_dict()


    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Union[Any, str]]:
    if response.status_code == 202:
        response_202 = cast(str, response.json())
        return response_202
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Union[Any, str]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    workflow_id: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: WorkflowExecutionRequest,

) -> Response[Union[Any, str]]:
    """ Execute a saved workflow for an agent

    Args:
        workflow_id (UUID):
        body (WorkflowExecutionRequest): Dynamic values to be merged into the saved workflow
            configuration. Example: {'name': 'Alice', 'model': 'gpt-4o'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, str]]
     """


    kwargs = _get_kwargs(
        workflow_id=workflow_id,
body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    workflow_id: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: WorkflowExecutionRequest,

) -> Optional[Union[Any, str]]:
    """ Execute a saved workflow for an agent

    Args:
        workflow_id (UUID):
        body (WorkflowExecutionRequest): Dynamic values to be merged into the saved workflow
            configuration. Example: {'name': 'Alice', 'model': 'gpt-4o'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, str]
     """


    result = sync_detailed(
        workflow_id=workflow_id,
client=client,
body=body,

    ).parsed

    return result

async def asyncio_detailed(
    workflow_id: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: WorkflowExecutionRequest,

) -> Response[Union[Any, str]]:
    """ Execute a saved workflow for an agent

    Args:
        workflow_id (UUID):
        body (WorkflowExecutionRequest): Dynamic values to be merged into the saved workflow
            configuration. Example: {'name': 'Alice', 'model': 'gpt-4o'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, str]]
     """


    kwargs = _get_kwargs(
        workflow_id=workflow_id,
body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    workflow_id: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: WorkflowExecutionRequest,

) -> Optional[Union[Any, str]]:
    """ Execute a saved workflow for an agent

    Args:
        workflow_id (UUID):
        body (WorkflowExecutionRequest): Dynamic values to be merged into the saved workflow
            configuration. Example: {'name': 'Alice', 'model': 'gpt-4o'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, str]
     """


    return (await asyncio_detailed(
        workflow_id=workflow_id,
client=client,
body=body,

    )).parsed



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.failure_category import FailureCategory
from ..models.failure_report_severity import FailureReportSeverity
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import cast, List
from typing import Dict
from typing import Union
from uuid import UUID
import datetime

if TYPE_CHECKING:
  from ..models.supervision_result import SupervisionResult
  from ..models.failure_report_tool_context import FailureReportToolContext





T = TypeVar("T", bound="FailureReport")


@_attrs_define
class FailureReport:
    """ Detailed information about a specific failure

        Attributes:
            run_id (UUID):
            timestamp (datetime.datetime):
            failure_category (FailureCategory):
            failure_reason (str): Detailed explanation of the failure
            supervisor_decisions (List['SupervisionResult']):
            severity (FailureReportSeverity):
            tool_context (Union[Unset, FailureReportToolContext]):
            remediation_suggestion (Union[Unset, str]): Suggested action to prevent similar failures
     """

    run_id: UUID
    timestamp: datetime.datetime
    failure_category: FailureCategory
    failure_reason: str
    supervisor_decisions: List['SupervisionResult']
    severity: FailureReportSeverity
    tool_context: Union[Unset, 'FailureReportToolContext'] = UNSET
    remediation_suggestion: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.supervision_result import SupervisionResult
        from ..models.failure_report_tool_context import FailureReportToolContext
        run_id = str(self.run_id)

        timestamp = self.timestamp.isoformat()

        failure_category = self.failure_category.value

        failure_reason = self.failure_reason

        supervisor_decisions = []
        for supervisor_decisions_item_data in self.supervisor_decisions:
            supervisor_decisions_item = supervisor_decisions_item_data.to_dict()
            supervisor_decisions.append(supervisor_decisions_item)



        severity = self.severity.value

        tool_context: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tool_context, Unset):
            tool_context = self.tool_context.to_dict()

        remediation_suggestion = self.remediation_suggestion


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "run_id": run_id,
            "timestamp": timestamp,
            "failure_category": failure_category,
            "failure_reason": failure_reason,
            "supervisor_decisions": supervisor_decisions,
            "severity": severity,
        })
        if tool_context is not UNSET:
            field_dict["tool_context"] = tool_context
        if remediation_suggestion is not UNSET:
            field_dict["remediation_suggestion"] = remediation_suggestion

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.supervision_result import SupervisionResult
        from ..models.failure_report_tool_context import FailureReportToolContext
        d = src_dict.copy()
        run_id = UUID(d.pop("run_id"))




        timestamp = isoparse(d.pop("timestamp"))




        failure_category = FailureCategory(d.pop("failure_category"))




        failure_reason = d.pop("failure_reason")

        supervisor_decisions = []
        _supervisor_decisions = d.pop("supervisor_decisions")
        for supervisor_decisions_item_data in (_supervisor_decisions):
            supervisor_decisions_item = SupervisionResult.from_dict(supervisor_decisions_item_data)



            supervisor_decisions.append(supervisor_decisions_item)


        severity = FailureReportSeverity(d.pop("severity"))




        _tool_context = d.pop("tool_context", UNSET)
        tool_context: Union[Unset, FailureReportToolContext]
        if isinstance(_tool_context,  Unset):
            tool_context = UNSET
        else:
            tool_context = FailureReportToolContext.from_dict(_tool_context)




        remediation_suggestion = d.pop("remediation_suggestion", UNSET)

        failure_report = cls(
            run_id=run_id,
            timestamp=timestamp,
            failure_category=failure_category,
            failure_reason=failure_reason,
            supervisor_decisions=supervisor_decisions,
            severity=severity,
            tool_context=tool_context,
            remediation_suggestion=remediation_suggestion,
        )


        failure_report.additional_properties = d
        return failure_report

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.decision import Decision
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import Union
from uuid import UUID
import datetime






T = TypeVar("T", bound="SupervisionResult")


@_attrs_define
class SupervisionResult:
    """ 
        Attributes:
            supervision_request_id (UUID):
            created_at (datetime.datetime):
            decision (Decision):
            reasoning (str):
            id (Union[Unset, UUID]):
            toolcall_id (Union[Unset, UUID]):
     """

    supervision_request_id: UUID
    created_at: datetime.datetime
    decision: Decision
    reasoning: str
    id: Union[Unset, UUID] = UNSET
    toolcall_id: Union[Unset, UUID] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        supervision_request_id = str(self.supervision_request_id)

        created_at = self.created_at.isoformat()

        decision = self.decision.value

        reasoning = self.reasoning

        id: Union[Unset, str] = UNSET
        if not isinstance(self.id, Unset):
            id = str(self.id)

        toolcall_id: Union[Unset, str] = UNSET
        if not isinstance(self.toolcall_id, Unset):
            toolcall_id = str(self.toolcall_id)


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "supervision_request_id": supervision_request_id,
            "created_at": created_at,
            "decision": decision,
            "reasoning": reasoning,
        })
        if id is not UNSET:
            field_dict["id"] = id
        if toolcall_id is not UNSET:
            field_dict["toolcall_id"] = toolcall_id

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        supervision_request_id = UUID(d.pop("supervision_request_id"))




        created_at = isoparse(d.pop("created_at"))




        decision = Decision(d.pop("decision"))




        reasoning = d.pop("reasoning")

        _id = d.pop("id", UNSET)
        id: Union[Unset, UUID]
        if isinstance(_id,  Unset):
            id = UNSET
        else:
            id = UUID(_id)




        _toolcall_id = d.pop("toolcall_id", UNSET)
        toolcall_id: Union[Unset, UUID]
        if isinstance(_toolcall_id,  Unset):
            toolcall_id = UNSET
        else:
            toolcall_id = UUID(_toolcall_id)




        supervision_result = cls(
            supervision_request_id=supervision_request_id,
            created_at=created_at,
            decision=decision,
            reasoning=reasoning,
            id=id,
            toolcall_id=toolcall_id,
        )


        supervision_result.additional_properties = d
        return supervision_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import cast, List
from typing import Dict
from uuid import UUID

if TYPE_CHECKING:
  from ..models.supervisor import Supervisor





T = TypeVar("T", bound="SupervisorChain")


@_attrs_define
class SupervisorChain:
    """ 
        Attributes:
            chain_id (UUID):
            supervisors (List['Supervisor']):
     """

    chain_id: UUID
    supervisors: List['Supervisor']
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.supervisor import Supervisor
        chain_id = str(self.chain_id)

        supervisors = []
        for supervisors_item_data in self.supervisors:
            supervisors_item = supervisors_item_data.to_dict()
            supervisors.append(supervisors_item)




        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "chain_id": chain_id,
            "supervisors": supervisors,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.supervisor import Supervisor
        d = src_dict.copy()
        chain_id = UUID(d.pop("chain_id"))




        supervisors = []
        _supervisors = d.pop("supervisors")
        for supervisors_item_data in (_supervisors):
            supervisors_item = Supervisor.from_dict(supervisors_item_data)



            supervisors.append(supervisors_item)


        supervisor_chain = cls(
            chain_id=chain_id,
            supervisors=supervisors,
        )


        supervisor_chain.additional_properties = d
        return supervisor_chain

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Dict

if TYPE_CHECKING:
  from ..models.agent_report_supervisor_statistics_reviews_by_type import AgentReportSupervisorStatisticsReviewsByType





T = TypeVar("T", bound="AgentReportSupervisorStatistics")


@_attrs_define
class AgentReportSupervisorStatistics:
    """ 
        Attributes:
            total_reviews (int):
            reviews_by_type (AgentReportSupervisorStatisticsReviewsByType):
     """

    total_reviews: int
    reviews_by_type: 'AgentReportSupervisorStatisticsReviewsByType'
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.agent_report_supervisor_statistics_reviews_by_type import AgentReportSupervisorStatisticsReviewsByType
        total_reviews = self.total_reviews

        reviews_by_type = self.reviews_by_type.to_dict()


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "total_reviews": total_reviews,
            "reviews_by_type": reviews_by_type,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.agent_report_supervisor_statistics_reviews_by_type import AgentReportSupervisorStatisticsReviewsByType
        d = src_dict.copy()
        total_reviews = d.pop("total_reviews")

        reviews_by_type = AgentReportSupervisorStatisticsReviewsByType.from_dict(d.pop("reviews_by_type"))




        agent_report_supervisor_statistics = cls(
            total_reviews=total_reviews,
            reviews_by_type=reviews_by_type,
        )


        agent_report_supervisor_statistics.additional_properties = d
        return agent_report_supervisor_statistics

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import cast, Union
from typing import Union
from uuid import UUID
import datetime






T = TypeVar("T", bound="APIKey")


@_attrs_define
class APIKey:
    """ 
        Attributes:
            name (str):
            id (Union[Unset, UUID]):
            key (Union[Unset, str]):
            user_id (Union[Unset, UUID]):
            created_at (Union[Unset, datetime.datetime]):
            expires_at (Union[Unset, datetime.datetime]):
            revoked_at (Union[None, Unset, datetime.datetime]):
     """

    name: str
    id: Union[Unset, UUID] = UNSET
    key: Union[Unset, str] = UNSET
    user_id: Union[Unset, UUID] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    expires_at: Union[Unset, datetime.datetime] = UNSET
    revoked_at: Union[None, Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        id: Union[Unset, str] = UNSET
        if not isinstance(self.id, Unset):
            id = str(self.id)

        key = self.key

        user_id: Union[Unset, str] = UNSET
        if not isinstance(self.user_id, Unset):
            user_id = str(self.user_id)

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        expires_at: Union[Unset, str] = UNSET
        if not isinstance(self.expires_at, Unset):
            expires_at = self.expires_at.isoformat()

        revoked_at: Union[None, Unset, str]
        if isinstance(self.revoked_at, Unset):
            revoked_at = UNSET
        elif isinstance(self.revoked_at, datetime.datetime):
            revoked_at = self.revoked_at.isoformat()
        else:
            revoked_at = self.revoked_at


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "name": name,
        })
        if id is not UNSET:
            field_dict["id"] = id
        if key is not UNSET:
            field_dict["key"] = key
        if user_id is not UNSET:
            field_dict["user_id"] = user_id
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if expires_at is not UNSET:
            field_dict["expires_at"] = expires_at
        if revoked_at is not UNSET:
            field_dict["revoked_at"] = revoked_at

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        _id = d.pop("id", UNSET)
        id: Union[Unset, UUID]
        if isinstance(_id,  Unset):
            id = UNSET
        else:
            id = UUID(_id)




        key = d.pop("key", UNSET)

        _user_id = d.pop("user_id", UNSET)
        user_id: Union[Unset, UUID]
        if isinstance(_user_id,  Unset):
            user_id = UNSET
        else:
            user_id = UUID(_user_id)




        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at,  Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)




        _expires_at = d.pop("expires_at", UNSET)
        expires_at: Union[Unset, datetime.datetime]
        if isinstance(_expires_at,  Unset):
            expires_at = UNSET
        else:
            expires_at = isoparse(_expires_at)




        def _parse_revoked_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                revoked_at_type_0 = isoparse(data)



                return revoked_at_type_0
            except: # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        revoked_at = _parse_revoked_at(d.pop("revoked_at", UNSET))


        api_key = cls(
            name=name,
            id=id,
            key=key,
            user_id=user_id,
            created_at=created_at,
            expires_at=expires_at,
            revoked_at=revoked_at,
        )


        api_key.additional_properties = d
        return api_key

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Dict

if TYPE_CHECKING:
  from ..models.asteroid_tool_call import AsteroidToolCall





T = TypeVar("T", bound="FailureReportToolContext")


@_attrs_define
class FailureReportToolContext:
    """ 
        Attributes:
            tool_name (str):
            tool_call (AsteroidToolCall):
     """

    tool_name: str
    tool_call: 'AsteroidToolCall'
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.asteroid_tool_call import AsteroidToolCall
        tool_name = self.tool_name

        tool_call = self.tool_call.to_dict()


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "tool_name": tool_name,
            "tool_call": tool_call,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.asteroid_tool_call import AsteroidToolCall
        d = src_dict.copy()
        tool_name = d.pop("tool_name")

        tool_call = AsteroidToolCall.from_dict(d.pop("tool_call"))




        failure_report_tool_context = cls(
            tool_name=tool_name,
            tool_call=tool_call,
        )


        failure_report_tool_context.additional_properties = d
        return failure_report_tool_context

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum

class FailureReportSeverity(str, Enum):
    CRITICAL = "critical"
    HIGH = "high"
    LOW = "low"
    MEDIUM = "medium"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import Union
from uuid import UUID






T = TypeVar("T", bound="User")


@_attrs_define
class User:
    """ 
        Attributes:
            id (Union[Unset, UUID]):
            email (Union[Unset, str]):
     """

    id: Union[Unset, UUID] = UNSET
    email: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        id: Union[Unset, str] = UNSET
        if not isinstance(self.id, Unset):
            id = str(self.id)

        email = self.email


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if id is not UNSET:
            field_dict["id"] = id
        if email is not UNSET:
            field_dict["email"] = email

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _id = d.pop("id", UNSET)
        id: Union[Unset, UUID]
        if isinstance(_id,  Unset):
            id = UNSET
        else:
            id = UUID(_id)




        email = d.pop("email", UNSET)

        user = cls(
            id=id,
            email=email,
        )


        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.asteroid_message_role import AsteroidMessageRole
from ..models.message_type import MessageType
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import cast, List
from typing import Dict
from typing import Union
from uuid import UUID
import datetime

if TYPE_CHECKING:
  from ..models.asteroid_tool_call import AsteroidToolCall





T = TypeVar("T", bound="AsteroidMessage")


@_attrs_define
class AsteroidMessage:
    """ 
        Attributes:
            role (AsteroidMessageRole):
            content (str):
            id (Union[Unset, UUID]):
            tool_calls (Union[Unset, List['AsteroidToolCall']]):
            type (Union[Unset, MessageType]):
            created_at (Union[Unset, datetime.datetime]):
            data (Union[Unset, str]): The raw b64 encoded JSON of the message objects in its original form
     """

    role: AsteroidMessageRole
    content: str
    id: Union[Unset, UUID] = UNSET
    tool_calls: Union[Unset, List['AsteroidToolCall']] = UNSET
    type: Union[Unset, MessageType] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    data: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.asteroid_tool_call import AsteroidToolCall
        role = self.role.value

        content = self.content

        id: Union[Unset, str] = UNSET
        if not isinstance(self.id, Unset):
            id = str(self.id)

        tool_calls: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.tool_calls, Unset):
            tool_calls = []
            for tool_calls_item_data in self.tool_calls:
                tool_calls_item = tool_calls_item_data.to_dict()
                tool_calls.append(tool_calls_item)



        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value


        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        data = self.data


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "role": role,
            "content": content,
        })
        if id is not UNSET:
            field_dict["id"] = id
        if tool_calls is not UNSET:
            field_dict["tool_calls"] = tool_calls
        if type is not UNSET:
            field_dict["type"] = type
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if data is not UNSET:
            field_dict["data"] = data

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.asteroid_tool_call import AsteroidToolCall
        d = src_dict.copy()
        role = AsteroidMessageRole(d.pop("role"))




        content = d.pop("content")

        _id = d.pop("id", UNSET)
        id: Union[Unset, UUID]
        if isinstance(_id,  Unset):
            id = UNSET
        else:
            id = UUID(_id)




        tool_calls = []
        _tool_calls = d.pop("tool_calls", UNSET)
        for tool_calls_item_data in (_tool_calls or []):
            tool_calls_item = AsteroidToolCall.from_dict(tool_calls_item_data)



            tool_calls.append(tool_calls_item)


        _type = d.pop("type", UNSET)
        type: Union[Unset, MessageType]
        if isinstance(_type,  Unset):
            type = UNSET
        else:
            type = MessageType(_type)




        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at,  Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)




        data = d.pop("data", UNSET)

        asteroid_message = cls(
            role=role,
            content=content,
            id=id,
            tool_calls=tool_calls,
            type=type,
            created_at=created_at,
            data=data,
        )


        asteroid_message.additional_properties = d
        return asteroid_message

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.create_workflow_request_provider import CreateWorkflowRequestProvider
from typing import cast
from typing import cast, List
from typing import Dict

if TYPE_CHECKING:
  from ..models.create_workflow_request_fields import CreateWorkflowRequestFields





T = TypeVar("T", bound="CreateWorkflowRequest")


@_attrs_define
class CreateWorkflowRequest:
    """ 
        Attributes:
            name (str): The name of the workflow. Example: My workflow.
            fields (CreateWorkflowRequestFields): JSON object containing static workflow configuration (e.g. a
                prompt_template). Example: {'model': 'gpt-4o', 'version': '2024-02-01'}.
            prompts (List[str]): The prompts for the workflow. They can have variables in them. They will be merged with the
                dynamic data passed when the workflow is executed. Example: ['Your name is {{.name}}, you speak {{.language}}',
                'Your task is {{.task}}'].
            provider (CreateWorkflowRequestProvider): The Language Model Provider for the Workflow Example: openai.
     """

    name: str
    fields: 'CreateWorkflowRequestFields'
    prompts: List[str]
    provider: CreateWorkflowRequestProvider
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.create_workflow_request_fields import CreateWorkflowRequestFields
        name = self.name

        fields = self.fields.to_dict()

        prompts = self.prompts



        provider = self.provider.value


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "name": name,
            "fields": fields,
            "prompts": prompts,
            "provider": provider,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_workflow_request_fields import CreateWorkflowRequestFields
        d = src_dict.copy()
        name = d.pop("name")

        fields = CreateWorkflowRequestFields.from_dict(d.pop("fields"))




        prompts = cast(List[str], d.pop("prompts"))


        provider = CreateWorkflowRequestProvider(d.pop("provider"))




        create_workflow_request = cls(
            name=name,
            fields=fields,
            prompts=prompts,
            provider=provider,
        )


        create_workflow_request.additional_properties = d
        return create_workflow_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast, List






T = TypeVar("T", bound="Agent")


@_attrs_define
class Agent:
    """ 
        Attributes:
            name (str): The name of the agent Example: my_agent.
            description (str): The description of the agent Example: This agent is used to queue jobs.
            visibility (str): The visibility of the agent Example: public.
            required_fields (List[str]): The required fields for the agent Example: ['system_prompt_template'].
            required_prompts (List[str]): The prompts for the agent Example: ['Hello {{.name}}, your model is {{.model}}'].
     """

    name: str
    description: str
    visibility: str
    required_fields: List[str]
    required_prompts: List[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        description = self.description

        visibility = self.visibility

        required_fields = self.required_fields



        required_prompts = self.required_prompts




        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "name": name,
            "description": description,
            "visibility": visibility,
            "required_fields": required_fields,
            "required_prompts": required_prompts,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        description = d.pop("description")

        visibility = d.pop("visibility")

        required_fields = cast(List[str], d.pop("required_fields"))


        required_prompts = cast(List[str], d.pop("required_prompts"))


        agent = cls(
            name=name,
            description=description,
            visibility=visibility,
            required_fields=required_fields,
            required_prompts=required_prompts,
        )


        agent.additional_properties = d
        return agent

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import cast, List
from typing import Dict
from typing import Union
from uuid import UUID

if TYPE_CHECKING:
  from ..models.tool_attributes import ToolAttributes





T = TypeVar("T", bound="Tool")


@_attrs_define
class Tool:
    """ 
        Attributes:
            run_id (UUID):
            name (str):
            description (str):
            attributes (ToolAttributes):
            code (str):
            id (Union[Unset, UUID]):
            ignored_attributes (Union[Unset, List[str]]):
     """

    run_id: UUID
    name: str
    description: str
    attributes: 'ToolAttributes'
    code: str
    id: Union[Unset, UUID] = UNSET
    ignored_attributes: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.tool_attributes import ToolAttributes
        run_id = str(self.run_id)

        name = self.name

        description = self.description

        attributes = self.attributes.to_dict()

        code = self.code

        id: Union[Unset, str] = UNSET
        if not isinstance(self.id, Unset):
            id = str(self.id)

        ignored_attributes: Union[Unset, List[str]] = UNSET
        if not isinstance(self.ignored_attributes, Unset):
            ignored_attributes = self.ignored_attributes




        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "run_id": run_id,
            "name": name,
            "description": description,
            "attributes": attributes,
            "code": code,
        })
        if id is not UNSET:
            field_dict["id"] = id
        if ignored_attributes is not UNSET:
            field_dict["ignored_attributes"] = ignored_attributes

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.tool_attributes import ToolAttributes
        d = src_dict.copy()
        run_id = UUID(d.pop("run_id"))




        name = d.pop("name")

        description = d.pop("description")

        attributes = ToolAttributes.from_dict(d.pop("attributes"))




        code = d.pop("code")

        _id = d.pop("id", UNSET)
        id: Union[Unset, UUID]
        if isinstance(_id,  Unset):
            id = UNSET
        else:
            id = UUID(_id)




        ignored_attributes = cast(List[str], d.pop("ignored_attributes", UNSET))


        tool = cls(
            run_id=run_id,
            name=name,
            description=description,
            attributes=attributes,
            code=code,
            id=id,
            ignored_attributes=ignored_attributes,
        )


        tool.additional_properties = d
        return tool

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.status import Status
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import Dict
from typing import Union
from uuid import UUID
import datetime

if TYPE_CHECKING:
  from ..models.run_metadata import RunMetadata





T = TypeVar("T", bound="Run")


@_attrs_define
class Run:
    """ 
        Attributes:
            id (UUID):
            task_id (UUID):
            created_at (datetime.datetime):
            status (Union[Unset, Status]):
            result (Union[Unset, str]):
            metadata (Union[Unset, RunMetadata]):
     """

    id: UUID
    task_id: UUID
    created_at: datetime.datetime
    status: Union[Unset, Status] = UNSET
    result: Union[Unset, str] = UNSET
    metadata: Union[Unset, 'RunMetadata'] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.run_metadata import RunMetadata
        id = str(self.id)

        task_id = str(self.task_id)

        created_at = self.created_at.isoformat()

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value


        result = self.result

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "task_id": task_id,
            "created_at": created_at,
        })
        if status is not UNSET:
            field_dict["status"] = status
        if result is not UNSET:
            field_dict["result"] = result
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.run_metadata import RunMetadata
        d = src_dict.copy()
        id = UUID(d.pop("id"))




        task_id = UUID(d.pop("task_id"))




        created_at = isoparse(d.pop("created_at"))




        _status = d.pop("status", UNSET)
        status: Union[Unset, Status]
        if isinstance(_status,  Unset):
            status = UNSET
        else:
            status = Status(_status)




        result = d.pop("result", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, RunMetadata]
        if isinstance(_metadata,  Unset):
            metadata = UNSET
        else:
            metadata = RunMetadata.from_dict(_metadata)




        run = cls(
            id=id,
            task_id=task_id,
            created_at=created_at,
            status=status,
            result=result,
            metadata=metadata,
        )


        run.additional_properties = d
        return run

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum

class SupervisorType(str, Enum):
    CLIENT_SUPERVISOR = "client_supervisor"
    HUMAN_SUPERVISOR = "human_supervisor"
    NO_SUPERVISOR = "no_supervisor"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum

class MessageType(str, Enum):
    AUDIO = "audio"
    IMAGE = "image"
    IMAGE_URL = "image_url"
    TEXT = "text"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Dict

if TYPE_CHECKING:
  from ..models.supervisor_stats_decisions import SupervisorStatsDecisions





T = TypeVar("T", bound="SupervisorStats")


@_attrs_define
class SupervisorStats:
    """ Statistics for a specific type of supervisor

        Attributes:
            total_reviews (int):
            decisions (SupervisorStatsDecisions):
            average_review_time (float): Average time taken for reviews in seconds
            reliability_score (float): Score indicating the reliability of this supervisor type (0-1)
     """

    total_reviews: int
    decisions: 'SupervisorStatsDecisions'
    average_review_time: float
    reliability_score: float
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.supervisor_stats_decisions import SupervisorStatsDecisions
        total_reviews = self.total_reviews

        decisions = self.decisions.to_dict()

        average_review_time = self.average_review_time

        reliability_score = self.reliability_score


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "total_reviews": total_reviews,
            "decisions": decisions,
            "average_review_time": average_review_time,
            "reliability_score": reliability_score,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.supervisor_stats_decisions import SupervisorStatsDecisions
        d = src_dict.copy()
        total_reviews = d.pop("total_reviews")

        decisions = SupervisorStatsDecisions.from_dict(d.pop("decisions"))




        average_review_time = d.pop("average_review_time")

        reliability_score = d.pop("reliability_score")

        supervisor_stats = cls(
            total_reviews=total_reviews,
            decisions=decisions,
            average_review_time=average_review_time,
            reliability_score=reliability_score,
        )


        supervisor_stats.additional_properties = d
        return supervisor_stats

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.permission import Permission
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import cast, List
from typing import Union
from uuid import UUID
import datetime






T = TypeVar("T", bound="Project")


@_attrs_define
class Project:
    """ 
        Attributes:
            id (UUID):
            name (str):
            created_at (datetime.datetime):
            run_result_tags (List[str]):
            permissions (Union[Unset, Permission]):
     """

    id: UUID
    name: str
    created_at: datetime.datetime
    run_result_tags: List[str]
    permissions: Union[Unset, Permission] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        id = str(self.id)

        name = self.name

        created_at = self.created_at.isoformat()

        run_result_tags = self.run_result_tags



        permissions: Union[Unset, str] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = self.permissions.value



        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "name": name,
            "created_at": created_at,
            "run_result_tags": run_result_tags,
        })
        if permissions is not UNSET:
            field_dict["permissions"] = permissions

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = UUID(d.pop("id"))




        name = d.pop("name")

        created_at = isoparse(d.pop("created_at"))




        run_result_tags = cast(List[str], d.pop("run_result_tags"))


        _permissions = d.pop("permissions", UNSET)
        permissions: Union[Unset, Permission]
        if isinstance(_permissions,  Unset):
            permissions = UNSET
        else:
            permissions = Permission(_permissions)




        project = cls(
            id=id,
            name=name,
            created_at=created_at,
            run_result_tags=run_result_tags,
            permissions=permissions,
        )


        project.additional_properties = d
        return project

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import cast, List
from typing import Dict
from typing import Union
from uuid import UUID

if TYPE_CHECKING:
  from ..models.agent_report_supervisor_statistics import AgentReportSupervisorStatistics
  from ..models.agent_report_time_period import AgentReportTimePeriod
  from ..models.agent_report_failure_categories import AgentReportFailureCategories
  from ..models.failure_report import FailureReport
  from ..models.agent_report_run_statistics import AgentReportRunStatistics





T = TypeVar("T", bound="AgentReport")


@_attrs_define
class AgentReport:
    """ A comprehensive report on an agent's performance across multiple runs

        Attributes:
            agent_id (UUID): Unique identifier for the agent
            total_runs (int): Total number of runs performed by the agent
            success_rate (float): Percentage of successful runs (0-100)
            run_statistics (AgentReportRunStatistics):
            failure_analysis (List['FailureReport']):
            failure_categories (AgentReportFailureCategories): Map of failure categories to their occurrence count
            supervisor_statistics (AgentReportSupervisorStatistics):
            time_period (AgentReportTimePeriod):
            prompt_text (Union[Unset, str]): The text of the prompt used by the agent
     """

    agent_id: UUID
    total_runs: int
    success_rate: float
    run_statistics: 'AgentReportRunStatistics'
    failure_analysis: List['FailureReport']
    failure_categories: 'AgentReportFailureCategories'
    supervisor_statistics: 'AgentReportSupervisorStatistics'
    time_period: 'AgentReportTimePeriod'
    prompt_text: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.agent_report_supervisor_statistics import AgentReportSupervisorStatistics
        from ..models.agent_report_time_period import AgentReportTimePeriod
        from ..models.agent_report_failure_categories import AgentReportFailureCategories
        from ..models.failure_report import FailureReport
        from ..models.agent_report_run_statistics import AgentReportRunStatistics
        agent_id = str(self.agent_id)

        total_runs = self.total_runs

        success_rate = self.success_rate

        run_statistics = self.run_statistics.to_dict()

        failure_analysis = []
        for failure_analysis_item_data in self.failure_analysis:
            failure_analysis_item = failure_analysis_item_data.to_dict()
            failure_analysis.append(failure_analysis_item)



        failure_categories = self.failure_categories.to_dict()

        supervisor_statistics = self.supervisor_statistics.to_dict()

        time_period = self.time_period.to_dict()

        prompt_text = self.prompt_text


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "agent_id": agent_id,
            "total_runs": total_runs,
            "success_rate": success_rate,
            "run_statistics": run_statistics,
            "failure_analysis": failure_analysis,
            "failure_categories": failure_categories,
            "supervisor_statistics": supervisor_statistics,
            "time_period": time_period,
        })
        if prompt_text is not UNSET:
            field_dict["prompt_text"] = prompt_text

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.agent_report_supervisor_statistics import AgentReportSupervisorStatistics
        from ..models.agent_report_time_period import AgentReportTimePeriod
        from ..models.agent_report_failure_categories import AgentReportFailureCategories
        from ..models.failure_report import FailureReport
        from ..models.agent_report_run_statistics import AgentReportRunStatistics
        d = src_dict.copy()
        agent_id = UUID(d.pop("agent_id"))




        total_runs = d.pop("total_runs")

        success_rate = d.pop("success_rate")

        run_statistics = AgentReportRunStatistics.from_dict(d.pop("run_statistics"))




        failure_analysis = []
        _failure_analysis = d.pop("failure_analysis")
        for failure_analysis_item_data in (_failure_analysis):
            failure_analysis_item = FailureReport.from_dict(failure_analysis_item_data)



            failure_analysis.append(failure_analysis_item)


        failure_categories = AgentReportFailureCategories.from_dict(d.pop("failure_categories"))




        supervisor_statistics = AgentReportSupervisorStatistics.from_dict(d.pop("supervisor_statistics"))




        time_period = AgentReportTimePeriod.from_dict(d.pop("time_period"))




        prompt_text = d.pop("prompt_text", UNSET)

        agent_report = cls(
            agent_id=agent_id,
            total_runs=total_runs,
            success_rate=success_rate,
            run_statistics=run_statistics,
            failure_analysis=failure_analysis,
            failure_categories=failure_categories,
            supervisor_statistics=supervisor_statistics,
            time_period=time_period,
            prompt_text=prompt_text,
        )


        agent_report.additional_properties = d
        return agent_report

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import cast, List
from typing import Dict

if TYPE_CHECKING:
  from ..models.workflow import Workflow
  from ..models.execution import Execution





T = TypeVar("T", bound="WorkflowExecution")


@_attrs_define
class WorkflowExecution:
    """ 
        Attributes:
            workflow (Workflow):
            executions (List['Execution']):
     """

    workflow: 'Workflow'
    executions: List['Execution']
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.workflow import Workflow
        from ..models.execution import Execution
        workflow = self.workflow.to_dict()

        executions = []
        for executions_item_data in self.executions:
            executions_item = executions_item_data.to_dict()
            executions.append(executions_item)




        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "workflow": workflow,
            "executions": executions,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.workflow import Workflow
        from ..models.execution import Execution
        d = src_dict.copy()
        workflow = Workflow.from_dict(d.pop("workflow"))




        executions = []
        _executions = d.pop("executions")
        for executions_item_data in (_executions):
            executions_item = Execution.from_dict(executions_item_data)



            executions.append(executions_item)


        workflow_execution = cls(
            workflow=workflow,
            executions=executions,
        )


        workflow_execution.additional_properties = d
        return workflow_execution

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.error_response import ErrorResponse
from ...models.review_payload import ReviewPayload
from typing import cast
from typing import Dict
from uuid import UUID



def _get_kwargs(
    supervision_request_id: UUID,

) -> Dict[str, Any]:
    

    

    

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/supervision_request/{supervision_request_id}/review_payload".format(supervision_request_id=supervision_request_id,),
    }


    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Union[ErrorResponse, ReviewPayload]]:
    if response.status_code == 200:
        response_200 = ReviewPayload.from_dict(response.json())



        return response_200
    if response.status_code == 404:
        response_404 = ErrorResponse.from_dict(response.json())



        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Union[ErrorResponse, ReviewPayload]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    supervision_request_id: UUID,
    *,
    client: Union[AuthenticatedClient, Client],

) -> Response[Union[ErrorResponse, ReviewPayload]]:
    """ Get the review payload for a supervision request

    Args:
        supervision_request_id (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ErrorResponse, ReviewPayload]]
     """


    kwargs = _get_kwargs(
        supervision_request_id=supervision_request_id,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    supervision_request_id: UUID,
    *,
    client: Union[AuthenticatedClient, Client],

) -> Optional[Union[ErrorResponse, ReviewPayload]]:
    """ Get the review payload for a supervision request

    Args:
        supervision_request_id (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ErrorResponse, ReviewPayload]
     """


    return sync_detailed(
        supervision_request_id=supervision_request_id,
client=client,

    ).parsed

async def asyncio_detailed(
    supervision_request_id: UUID,
    *,
    client: Union[AuthenticatedClient, Client],

) -> Response[Union[ErrorResponse, ReviewPayload]]:
    """ Get the review payload for a supervision request

    Args:
        supervision_request_id (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ErrorResponse, ReviewPayload]]
     """


    kwargs = _get_kwargs(
        supervision_request_id=supervision_request_id,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    supervision_request_id: UUID,
    *,
    client: Union[AuthenticatedClient, Client],

) -> Optional[Union[ErrorResponse, ReviewPayload]]:
    """ Get the review payload for a supervision request

    Args:
        supervision_request_id (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ErrorResponse, ReviewPayload]
     """


    return (await asyncio_detailed(
        supervision_request_id=supervision_request_id,
client=client,

    )).parsed



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.status import Status
from typing import cast
from typing import cast, List
from typing import Dict

if TYPE_CHECKING:
  from ..models.asteroid_tool_call import AsteroidToolCall
  from ..models.chain_execution_state import ChainExecutionState





T = TypeVar("T", bound="RunExecution")


@_attrs_define
class RunExecution:
    """ 
        Attributes:
            toolcall (AsteroidToolCall):
            chains (List['ChainExecutionState']):
            status (Status):
     """

    toolcall: 'AsteroidToolCall'
    chains: List['ChainExecutionState']
    status: Status
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.asteroid_tool_call import AsteroidToolCall
        from ..models.chain_execution_state import ChainExecutionState
        toolcall = self.toolcall.to_dict()

        chains = []
        for chains_item_data in self.chains:
            chains_item = chains_item_data.to_dict()
            chains.append(chains_item)



        status = self.status.value


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "toolcall": toolcall,
            "chains": chains,
            "status": status,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.asteroid_tool_call import AsteroidToolCall
        from ..models.chain_execution_state import ChainExecutionState
        d = src_dict.copy()
        toolcall = AsteroidToolCall.from_dict(d.pop("toolcall"))




        chains = []
        _chains = d.pop("chains")
        for chains_item_data in (_chains):
            chains_item = ChainExecutionState.from_dict(chains_item_data)



            chains.append(chains_item)


        status = Status(d.pop("status"))




        run_execution = cls(
            toolcall=toolcall,
            chains=chains,
            status=status,
        )


        run_execution.additional_properties = d
        return run_execution

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import Union






T = TypeVar("T", bound="CreateTaskBody")


@_attrs_define
class CreateTaskBody:
    """ 
        Attributes:
            name (str):
            description (Union[Unset, str]):
     """

    name: str
    description: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        description = self.description


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "name": name,
        })
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        description = d.pop("description", UNSET)

        create_task_body = cls(
            name=name,
            description=description,
        )


        create_task_body.additional_properties = d
        return create_task_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.supervisor_type import SupervisorType
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import Dict
from typing import Union
from uuid import UUID
import datetime

if TYPE_CHECKING:
  from ..models.supervisor_attributes import SupervisorAttributes





T = TypeVar("T", bound="Supervisor")


@_attrs_define
class Supervisor:
    """ 
        Attributes:
            name (str):
            description (str):
            created_at (datetime.datetime):
            type (SupervisorType): The type of supervisor. ClientSupervisor means that the supervision is done client side
                and the server is merely informed. Other supervisor types are handled serverside, e.g. HumanSupervisor means
                that a human will review the request via the Asteroid UI.
            code (str):
            attributes (SupervisorAttributes):
            id (Union[Unset, UUID]):
     """

    name: str
    description: str
    created_at: datetime.datetime
    type: SupervisorType
    code: str
    attributes: 'SupervisorAttributes'
    id: Union[Unset, UUID] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.supervisor_attributes import SupervisorAttributes
        name = self.name

        description = self.description

        created_at = self.created_at.isoformat()

        type = self.type.value

        code = self.code

        attributes = self.attributes.to_dict()

        id: Union[Unset, str] = UNSET
        if not isinstance(self.id, Unset):
            id = str(self.id)


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "name": name,
            "description": description,
            "created_at": created_at,
            "type": type,
            "code": code,
            "attributes": attributes,
        })
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.supervisor_attributes import SupervisorAttributes
        d = src_dict.copy()
        name = d.pop("name")

        description = d.pop("description")

        created_at = isoparse(d.pop("created_at"))




        type = SupervisorType(d.pop("type"))




        code = d.pop("code")

        attributes = SupervisorAttributes.from_dict(d.pop("attributes"))




        _id = d.pop("id", UNSET)
        id: Union[Unset, UUID]
        if isinstance(_id,  Unset):
            id = UNSET
        else:
            id = UUID(_id)




        supervisor = cls(
            name=name,
            description=description,
            created_at=created_at,
            type=type,
            code=code,
            attributes=attributes,
            id=id,
        )


        supervisor.additional_properties = d
        return supervisor

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .agent_report import AgentReport
from .agent_report_failure_categories import AgentReportFailureCategories
from .agent_report_run_statistics import AgentReportRunStatistics
from .agent_report_supervisor_statistics import AgentReportSupervisorStatistics
from .agent_report_supervisor_statistics_reviews_by_type import AgentReportSupervisorStatisticsReviewsByType
from .agent_report_time_period import AgentReportTimePeriod
from .api_key import APIKey
from .asteroid_chat import AsteroidChat
from .asteroid_choice import AsteroidChoice
from .asteroid_choice_finish_reason_type_1 import AsteroidChoiceFinishReasonType1
from .asteroid_message import AsteroidMessage
from .asteroid_message_role import AsteroidMessageRole
from .asteroid_tool_call import AsteroidToolCall
from .chain_execution import ChainExecution
from .chain_execution_state import ChainExecutionState
from .chain_request import ChainRequest
from .chat_format import ChatFormat
from .chat_ids import ChatIds
from .choice_ids import ChoiceIds
from .create_project_body import CreateProjectBody
from .create_run_body import CreateRunBody
from .create_run_tool_body import CreateRunToolBody
from .create_run_tool_body_attributes import CreateRunToolBodyAttributes
from .create_task_body import CreateTaskBody
from .decision import Decision
from .error_response import ErrorResponse
from .failure_category import FailureCategory
from .failure_report import FailureReport
from .failure_report_severity import FailureReportSeverity
from .failure_report_tool_context import FailureReportToolContext
from .feedback import Feedback
from .feedback_request import FeedbackRequest
from .file_url import FileURL
from .get_create_file_url_body import GetCreateFileURLBody
from .hub_stats import HubStats
from .hub_stats_assigned_reviews import HubStatsAssignedReviews
from .hub_stats_review_distribution import HubStatsReviewDistribution
from .message_role import MessageRole
from .message_type import MessageType
from .permission import Permission
from .project import Project
from .review_payload import ReviewPayload
from .revoke_api_key_body import RevokeAPIKeyBody
from .run import Run
from .run_execution import RunExecution
from .run_metadata import RunMetadata
from .status import Status
from .supervision_request import SupervisionRequest
from .supervision_request_state import SupervisionRequestState
from .supervision_result import SupervisionResult
from .supervision_status import SupervisionStatus
from .supervisor import Supervisor
from .supervisor_attributes import SupervisorAttributes
from .supervisor_chain import SupervisorChain
from .supervisor_stats import SupervisorStats
from .supervisor_stats_decisions import SupervisorStatsDecisions
from .supervisor_type import SupervisorType
from .task import Task
from .tool import Tool
from .tool_attributes import ToolAttributes
from .tool_call_ids import ToolCallIds
from .tool_supervision_result import ToolSupervisionResult
from .update_run_metadata_body import UpdateRunMetadataBody
from .update_run_result_body import UpdateRunResultBody
from .user import User

__all__ = (
    "AgentReport",
    "AgentReportFailureCategories",
    "AgentReportRunStatistics",
    "AgentReportSupervisorStatistics",
    "AgentReportSupervisorStatisticsReviewsByType",
    "AgentReportTimePeriod",
    "APIKey",
    "AsteroidChat",
    "AsteroidChoice",
    "AsteroidChoiceFinishReasonType1",
    "AsteroidMessage",
    "AsteroidMessageRole",
    "AsteroidToolCall",
    "ChainExecution",
    "ChainExecutionState",
    "ChainRequest",
    "ChatFormat",
    "ChatIds",
    "ChoiceIds",
    "CreateProjectBody",
    "CreateRunBody",
    "CreateRunToolBody",
    "CreateRunToolBodyAttributes",
    "CreateTaskBody",
    "Decision",
    "ErrorResponse",
    "FailureCategory",
    "FailureReport",
    "FailureReportSeverity",
    "FailureReportToolContext",
    "Feedback",
    "FeedbackRequest",
    "FileURL",
    "GetCreateFileURLBody",
    "HubStats",
    "HubStatsAssignedReviews",
    "HubStatsReviewDistribution",
    "MessageRole",
    "MessageType",
    "Permission",
    "Project",
    "ReviewPayload",
    "RevokeAPIKeyBody",
    "Run",
    "RunExecution",
    "RunMetadata",
    "Status",
    "SupervisionRequest",
    "SupervisionRequestState",
    "SupervisionResult",
    "SupervisionStatus",
    "Supervisor",
    "SupervisorAttributes",
    "SupervisorChain",
    "SupervisorStats",
    "SupervisorStatsDecisions",
    "SupervisorType",
    "Task",
    "Tool",
    "ToolAttributes",
    "ToolCallIds",
    "ToolSupervisionResult",
    "UpdateRunMetadataBody",
    "UpdateRunResultBody",
    "User",
)



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Dict

if TYPE_CHECKING:
  from ..models.supervisor_stats import SupervisorStats





T = TypeVar("T", bound="AgentReportSupervisorStatisticsReviewsByType")


@_attrs_define
class AgentReportSupervisorStatisticsReviewsByType:
    """ 
     """

    additional_properties: Dict[str, 'SupervisorStats'] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.supervisor_stats import SupervisorStats
        
        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.supervisor_stats import SupervisorStats
        d = src_dict.copy()
        agent_report_supervisor_statistics_reviews_by_type = cls(
        )


        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = SupervisorStats.from_dict(prop_dict)



            additional_properties[prop_name] = additional_property

        agent_report_supervisor_statistics_reviews_by_type.additional_properties = additional_properties
        return agent_report_supervisor_statistics_reviews_by_type

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> 'SupervisorStats':
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: 'SupervisorStats') -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .agent import Agent
from .create_workflow_request import CreateWorkflowRequest
from .create_workflow_request_fields import CreateWorkflowRequestFields
from .create_workflow_request_provider import CreateWorkflowRequestProvider
from .execution import Execution
from .execution_dynamic_data import ExecutionDynamicData
from .health_check_response_200 import HealthCheckResponse200
from .health_check_response_500 import HealthCheckResponse500
from .optimisation_request import OptimisationRequest
from .workflow import Workflow
from .workflow_execution import WorkflowExecution
from .workflow_execution_request import WorkflowExecutionRequest
from .workflow_fields import WorkflowFields

__all__ = (
    "Agent",
    "CreateWorkflowRequest",
    "CreateWorkflowRequestFields",
    "CreateWorkflowRequestProvider",
    "Execution",
    "ExecutionDynamicData",
    "HealthCheckResponse200",
    "HealthCheckResponse500",
    "OptimisationRequest",
    "Workflow",
    "WorkflowExecution",
    "WorkflowExecutionRequest",
    "WorkflowFields",
)



from enum import Enum

class FailureCategory(str, Enum):
    HALLUCINATION = "hallucination"
    INCORRECT_TOOL_USAGE = "incorrect_tool_usage"
    INVALID_OUTPUT_FORMAT = "invalid_output_format"
    LOGICAL_ERROR = "logical_error"
    OTHER = "other"
    PERMISSION_ERROR = "permission_error"
    RESOURCE_EXHAUSTION = "resource_exhaustion"
    RUNTIME_ERROR = "runtime_error"
    SAFETY_VIOLATION = "safety_violation"
    TIMEOUT = "timeout"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import Union
from uuid import UUID
import datetime






T = TypeVar("T", bound="AsteroidToolCall")


@_attrs_define
class AsteroidToolCall:
    """ 
        Attributes:
            id (UUID):
            tool_id (UUID):
            call_id (Union[Unset, str]):
            name (Union[Unset, str]):
            arguments (Union[Unset, str]): Arguments in JSON format
            created_at (Union[Unset, datetime.datetime]):
     """

    id: UUID
    tool_id: UUID
    call_id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    arguments: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        id = str(self.id)

        tool_id = str(self.tool_id)

        call_id = self.call_id

        name = self.name

        arguments = self.arguments

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "tool_id": tool_id,
        })
        if call_id is not UNSET:
            field_dict["call_id"] = call_id
        if name is not UNSET:
            field_dict["name"] = name
        if arguments is not UNSET:
            field_dict["arguments"] = arguments
        if created_at is not UNSET:
            field_dict["created_at"] = created_at

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = UUID(d.pop("id"))




        tool_id = UUID(d.pop("tool_id"))




        call_id = d.pop("call_id", UNSET)

        name = d.pop("name", UNSET)

        arguments = d.pop("arguments", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at,  Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)




        asteroid_tool_call = cls(
            id=id,
            tool_id=tool_id,
            call_id=call_id,
            name=name,
            arguments=arguments,
            created_at=created_at,
        )


        asteroid_tool_call.additional_properties = d
        return asteroid_tool_call

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum

class AsteroidChoiceFinishReasonType1(str, Enum):
    CONTENT_FILTER = "content_filter"
    FUNCTION_CALL = "function_call"
    LENGTH = "length"
    STOP = "stop"
    TOOL_CALLS = "tool_calls"

    def __str__(self) -> str:
        return str(self.value)



import logging
import os
from typing import Optional, List, Dict, Any
from uuid import UUID

from api.generated.agents.asteroid_agents_api_client.api.agent.get_agents import sync as get_agents_sync
from api.generated.agents.asteroid_agents_api_client.api.api.get_open_api import sync_detailed as get_open_api_sync
from api.generated.agents.asteroid_agents_api_client.api.default.create_workflow import sync as create_workflow_sync
from api.generated.agents.asteroid_agents_api_client.api.default.health_check import sync as health_check_sync
from api.generated.agents.asteroid_agents_api_client.api.workflow.execute_workflow import sync as execute_workflow_sync
from api.generated.agents.asteroid_agents_api_client.api.workflow.get_workflow_executions import \
    sync as get_workflow_executions_sync
from api.generated.agents.asteroid_agents_api_client.client import Client as AgentsClient
from api.generated.agents.asteroid_agents_api_client.models import CreateWorkflowRequest, WorkflowExecutionRequest, \
    WorkflowExecution
from api.generated.platform.asteroid_api_client.api.api_key.validate_api_key import \
    sync_detailed as validate_api_key_sync
from api.generated.platform.asteroid_api_client.api.improve.create_feedback import sync_detailed as create_feedback_sync
from api.generated.platform.asteroid_api_client.api.run.get_run import sync as get_run_sync
from api.generated.platform.asteroid_api_client.api.run.get_run_status import sync as get_run_status_sync
from api.generated.platform.asteroid_api_client.client import Client as PlatformClient
from api.generated.platform.asteroid_api_client.models import Status
from api.generated.platform.asteroid_api_client.models.error_response import ErrorResponse
from api.generated.platform.asteroid_api_client.models.feedback import Feedback
from api.generated.platform.asteroid_api_client.models.feedback_request import FeedbackRequest
from asteroid_odyssey.exceptions import ApiError

# Logger
logger = logging.getLogger(__name__)


class Odyssey:
    """Wrapper for the generated API client."""

    def __init__(
            self,
            api_key: Optional[str] = None,
            agents_base_url: Optional[str] = None,
            platform_base_url: Optional[str] = None
    ):
        """Initialize the client.

        Args:
            api_key: Optional API key for authentication
            agents_base_url: Base URL for the agents API
            platform_base_url: Base URL for the platform API
        """
        if api_key is None:
            api_key = os.getenv("ASTEROID_API_KEY")
            if not api_key:
                raise ApiError(
                    "Asteroid API key is required, please set the ASTEROID_API_KEY environment variable. You can get one from https://platform.asteroid.com/")

        if agents_base_url is None:
            from_env = os.getenv("ASTEROID_AGENTS_API_URL")
            if not from_env:
                from_env = "https://odyssey.asteroid.ai/api/v1"
            agents_base_url = from_env

        if platform_base_url is None:
            from_env = os.getenv("ASTEROID_API_URL")
            if not from_env:
                from_env = "https://platform.asteroid.com/api/v1"
            platform_base_url = from_env

        self._agents_client = AgentsClient(
            base_url=agents_base_url,
            verify_ssl=False,
            headers={"X-Asteroid-Agents-Api-Key": f"{api_key}"}
        )

        self._platform_client = PlatformClient(
            base_url=platform_base_url,
            verify_ssl=False,
            headers={"X-Asteroid-Api-Key": f"{api_key}"}
        )

        print(f"Validating API key")
        try:
            response = validate_api_key_sync(client=self._platform_client)
            print(f"Response: {response}")
        except Exception as e:
            print(f"Error validating API key: {e}")
            raise e

    def get_agents(self) -> List[Dict[str, Any]]:
        """Retrieves a list of all agents."""
        try:
            response = get_agents_sync(client=self._agents_client)
            return response
        except Exception as e:
            logger.error(f"Error retrieving agents: {e}")
            raise e

    def create_workflow(self, agent_name: str, request: CreateWorkflowRequest) -> str:
        """Creates a new workflow for a given agent."""
        try:
            workflow_id = create_workflow_sync(client=self._agents_client, agent_name=agent_name, body=request)
            return workflow_id
        except Exception as e:
            logger.error(f"Error creating workflow: {e}")
            raise e

    def run_workflow(self, workflow_id: UUID, request: WorkflowExecutionRequest) -> str:
        """Executes a saved workflow for an agent."""
        try:
            response = execute_workflow_sync(client=self._agents_client, workflow_id=workflow_id, body=request)
            return response
        except Exception as e:
            logger.error(f"Error running workflow: {e}")
            raise e

    def get_workflow_runs(self) -> list[WorkflowExecution] | None:
        """Retrieves all workflows along with their executions."""
        try:
            response = get_workflow_executions_sync(client=self._agents_client)
            return response
        except Exception as e:
            logger.error(f"Error retrieving workflow runs: {e}")
            raise e

    def create_run_feedback(self, run_id: UUID, request: FeedbackRequest) -> Feedback:
        """Creates feedback for a run."""
        try:
            response = create_feedback_sync(client=self._platform_client, run_id=run_id, body=request)
            return response
        except Exception as e:
            logger.error(f"Error creating feedback: {e}")
            raise e

    def get_open_api_schema(self) -> Any:
        """Retrieves the OpenAPI schema from the API."""
        try:
            response = get_open_api_sync(client=self._agents_client)
            return response
        except Exception as e:
            logger.error(f"Error retrieving OpenAPI schema: {e}")
            raise e

    def health_check(self) -> Dict[str, Any]:
        """Checks the health of the API."""
        try:
            response = health_check_sync(client=self._agents_client)
            return response
        except Exception as e:
            logger.error(f"Error performing health check: {e}")
            raise e

    def get_run_status(self, run_id: UUID) -> Status:
        """Retrieves the status of a run."""
        try:
            response = get_run_status_sync(client=self._platform_client, run_id=run_id)
            if isinstance(response, ErrorResponse):
                return None
            return response
        except Exception as e:
            logger.error(f"Error retrieving run status: {e}")
            raise e

    def get_run_result(self, run_id: UUID) -> str:
        """Retrieves the result of a run."""
        try:
            run = get_run_sync(client=self._platform_client, run_id=run_id)
            metadata = run.metadata
            if not metadata:
                raise ApiError("Run metadata not found")
            result = metadata.additional_properties.get('final_result')
            if not result:
                raise ApiError("Run result not found")
            return result
        except Exception as e:
            logger.error(f"Error retrieving run result: {e}")
            raise e



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
from typing import cast
from uuid import UUID
import datetime






T = TypeVar("T", bound="Feedback")


@_attrs_define
class Feedback:
    """ 
        Attributes:
            id (UUID):
            run_id (UUID):
            created_at (datetime.datetime): The timestamp of when the feedback was created
            updated_at (datetime.datetime): The timestamp of when the feedback was last updated
            feedback (str):
     """

    id: UUID
    run_id: UUID
    created_at: datetime.datetime
    updated_at: datetime.datetime
    feedback: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        id = str(self.id)

        run_id = str(self.run_id)

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        feedback = self.feedback


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "run_id": run_id,
            "created_at": created_at,
            "updated_at": updated_at,
            "feedback": feedback,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = UUID(d.pop("id"))




        run_id = UUID(d.pop("run_id"))




        created_at = isoparse(d.pop("created_at"))




        updated_at = isoparse(d.pop("updated_at"))




        feedback = d.pop("feedback")

        feedback = cls(
            id=id,
            run_id=run_id,
            created_at=created_at,
            updated_at=updated_at,
            feedback=feedback,
        )


        feedback.additional_properties = d
        return feedback

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import Dict
from typing import Union
from uuid import UUID

if TYPE_CHECKING:
  from ..models.execution_dynamic_data import ExecutionDynamicData





T = TypeVar("T", bound="Execution")


@_attrs_define
class Execution:
    """ 
        Attributes:
            id (Union[Unset, UUID]): Execution identifier.
            run_id (Union[Unset, UUID]): Run ID.
            dynamic_data (Union[Unset, ExecutionDynamicData]): Dynamic data to be merged into the saved workflow
                configuration. Example: {'name': 'Alice', 'model': 'gpt-4o'}.
            workflow_id (Union[Unset, UUID]): Workflow ID.
     """

    id: Union[Unset, UUID] = UNSET
    run_id: Union[Unset, UUID] = UNSET
    dynamic_data: Union[Unset, 'ExecutionDynamicData'] = UNSET
    workflow_id: Union[Unset, UUID] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.execution_dynamic_data import ExecutionDynamicData
        id: Union[Unset, str] = UNSET
        if not isinstance(self.id, Unset):
            id = str(self.id)

        run_id: Union[Unset, str] = UNSET
        if not isinstance(self.run_id, Unset):
            run_id = str(self.run_id)

        dynamic_data: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.dynamic_data, Unset):
            dynamic_data = self.dynamic_data.to_dict()

        workflow_id: Union[Unset, str] = UNSET
        if not isinstance(self.workflow_id, Unset):
            workflow_id = str(self.workflow_id)


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if id is not UNSET:
            field_dict["id"] = id
        if run_id is not UNSET:
            field_dict["run_id"] = run_id
        if dynamic_data is not UNSET:
            field_dict["dynamic_data"] = dynamic_data
        if workflow_id is not UNSET:
            field_dict["workflow_id"] = workflow_id

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.execution_dynamic_data import ExecutionDynamicData
        d = src_dict.copy()
        _id = d.pop("id", UNSET)
        id: Union[Unset, UUID]
        if isinstance(_id,  Unset):
            id = UNSET
        else:
            id = UUID(_id)




        _run_id = d.pop("run_id", UNSET)
        run_id: Union[Unset, UUID]
        if isinstance(_run_id,  Unset):
            run_id = UNSET
        else:
            run_id = UUID(_run_id)




        _dynamic_data = d.pop("dynamic_data", UNSET)
        dynamic_data: Union[Unset, ExecutionDynamicData]
        if isinstance(_dynamic_data,  Unset):
            dynamic_data = UNSET
        else:
            dynamic_data = ExecutionDynamicData.from_dict(_dynamic_data)




        _workflow_id = d.pop("workflow_id", UNSET)
        workflow_id: Union[Unset, UUID]
        if isinstance(_workflow_id,  Unset):
            workflow_id = UNSET
        else:
            workflow_id = UUID(_workflow_id)




        execution = cls(
            id=id,
            run_id=run_id,
            dynamic_data=dynamic_data,
            workflow_id=workflow_id,
        )


        execution.additional_properties = d
        return execution

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import Dict
from typing import Union
from uuid import UUID
import datetime

if TYPE_CHECKING:
  from ..models.supervision_status import SupervisionStatus





T = TypeVar("T", bound="SupervisionRequest")


@_attrs_define
class SupervisionRequest:
    """ 
        Attributes:
            supervisor_id (UUID):
            position_in_chain (int):
            id (Union[Unset, UUID]):
            chainexecution_id (Union[Unset, UUID]):
            status (Union[Unset, SupervisionStatus]):
            last_status_checked_at (Union[Unset, datetime.datetime]):
     """

    supervisor_id: UUID
    position_in_chain: int
    id: Union[Unset, UUID] = UNSET
    chainexecution_id: Union[Unset, UUID] = UNSET
    status: Union[Unset, 'SupervisionStatus'] = UNSET
    last_status_checked_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.supervision_status import SupervisionStatus
        supervisor_id = str(self.supervisor_id)

        position_in_chain = self.position_in_chain

        id: Union[Unset, str] = UNSET
        if not isinstance(self.id, Unset):
            id = str(self.id)

        chainexecution_id: Union[Unset, str] = UNSET
        if not isinstance(self.chainexecution_id, Unset):
            chainexecution_id = str(self.chainexecution_id)

        status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.to_dict()

        last_status_checked_at: Union[Unset, str] = UNSET
        if not isinstance(self.last_status_checked_at, Unset):
            last_status_checked_at = self.last_status_checked_at.isoformat()


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "supervisor_id": supervisor_id,
            "position_in_chain": position_in_chain,
        })
        if id is not UNSET:
            field_dict["id"] = id
        if chainexecution_id is not UNSET:
            field_dict["chainexecution_id"] = chainexecution_id
        if status is not UNSET:
            field_dict["status"] = status
        if last_status_checked_at is not UNSET:
            field_dict["last_status_checked_at"] = last_status_checked_at

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.supervision_status import SupervisionStatus
        d = src_dict.copy()
        supervisor_id = UUID(d.pop("supervisor_id"))




        position_in_chain = d.pop("position_in_chain")

        _id = d.pop("id", UNSET)
        id: Union[Unset, UUID]
        if isinstance(_id,  Unset):
            id = UNSET
        else:
            id = UUID(_id)




        _chainexecution_id = d.pop("chainexecution_id", UNSET)
        chainexecution_id: Union[Unset, UUID]
        if isinstance(_chainexecution_id,  Unset):
            chainexecution_id = UNSET
        else:
            chainexecution_id = UUID(_chainexecution_id)




        _status = d.pop("status", UNSET)
        status: Union[Unset, SupervisionStatus]
        if isinstance(_status,  Unset):
            status = UNSET
        else:
            status = SupervisionStatus.from_dict(_status)




        _last_status_checked_at = d.pop("last_status_checked_at", UNSET)
        last_status_checked_at: Union[Unset, datetime.datetime]
        if isinstance(_last_status_checked_at,  Unset):
            last_status_checked_at = UNSET
        else:
            last_status_checked_at = isoparse(_last_status_checked_at)




        supervision_request = cls(
            supervisor_id=supervisor_id,
            position_in_chain=position_in_chain,
            id=id,
            chainexecution_id=chainexecution_id,
            status=status,
            last_status_checked_at=last_status_checked_at,
        )


        supervision_request.additional_properties = d
        return supervision_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import Union






T = TypeVar("T", bound="AgentReportRunStatistics")


@_attrs_define
class AgentReportRunStatistics:
    """ 
        Attributes:
            successful_runs (Union[Unset, int]):
            failed_runs (Union[Unset, int]):
            pending_runs (Union[Unset, int]):
            average_run_duration (Union[Unset, float]): Average duration of runs in seconds
     """

    successful_runs: Union[Unset, int] = UNSET
    failed_runs: Union[Unset, int] = UNSET
    pending_runs: Union[Unset, int] = UNSET
    average_run_duration: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        successful_runs = self.successful_runs

        failed_runs = self.failed_runs

        pending_runs = self.pending_runs

        average_run_duration = self.average_run_duration


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if successful_runs is not UNSET:
            field_dict["successful_runs"] = successful_runs
        if failed_runs is not UNSET:
            field_dict["failed_runs"] = failed_runs
        if pending_runs is not UNSET:
            field_dict["pending_runs"] = pending_runs
        if average_run_duration is not UNSET:
            field_dict["average_run_duration"] = average_run_duration

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        successful_runs = d.pop("successful_runs", UNSET)

        failed_runs = d.pop("failed_runs", UNSET)

        pending_runs = d.pop("pending_runs", UNSET)

        average_run_duration = d.pop("average_run_duration", UNSET)

        agent_report_run_statistics = cls(
            successful_runs=successful_runs,
            failed_runs=failed_runs,
            pending_runs=pending_runs,
            average_run_duration=average_run_duration,
        )


        agent_report_run_statistics.additional_properties = d
        return agent_report_run_statistics

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.decision import Decision
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import Union
from uuid import UUID
import datetime






T = TypeVar("T", bound="ToolSupervisionResult")


@_attrs_define
class ToolSupervisionResult:
    """ 
        Attributes:
            id (UUID): The ID of the supervision result
            tool_id (UUID): The ID of the tool that the supervision result is for
            tool_call_id (UUID):
            supervisor_id (UUID): The ID of the supervisor that made the supervision result
            created_at (datetime.datetime): The timestamp of when the supervision result was created
            decision (Decision):
            reasoning (str): The reasoning behind the decision
            run_id (UUID): The ID of the run that the supervision result is for
            tool_name (Union[Unset, str]): The name of the tool that the supervision result is for
     """

    id: UUID
    tool_id: UUID
    tool_call_id: UUID
    supervisor_id: UUID
    created_at: datetime.datetime
    decision: Decision
    reasoning: str
    run_id: UUID
    tool_name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        id = str(self.id)

        tool_id = str(self.tool_id)

        tool_call_id = str(self.tool_call_id)

        supervisor_id = str(self.supervisor_id)

        created_at = self.created_at.isoformat()

        decision = self.decision.value

        reasoning = self.reasoning

        run_id = str(self.run_id)

        tool_name = self.tool_name


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "tool_id": tool_id,
            "tool_call_id": tool_call_id,
            "supervisor_id": supervisor_id,
            "created_at": created_at,
            "decision": decision,
            "reasoning": reasoning,
            "run_id": run_id,
        })
        if tool_name is not UNSET:
            field_dict["tool_name"] = tool_name

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = UUID(d.pop("id"))




        tool_id = UUID(d.pop("tool_id"))




        tool_call_id = UUID(d.pop("tool_call_id"))




        supervisor_id = UUID(d.pop("supervisor_id"))




        created_at = isoparse(d.pop("created_at"))




        decision = Decision(d.pop("decision"))




        reasoning = d.pop("reasoning")

        run_id = UUID(d.pop("run_id"))




        tool_name = d.pop("tool_name", UNSET)

        tool_supervision_result = cls(
            id=id,
            tool_id=tool_id,
            tool_call_id=tool_call_id,
            supervisor_id=supervisor_id,
            created_at=created_at,
            decision=decision,
            reasoning=reasoning,
            run_id=run_id,
            tool_name=tool_name,
        )


        tool_supervision_result.additional_properties = d
        return tool_supervision_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import cast, List
from typing import Dict
from typing import Union
from uuid import UUID

if TYPE_CHECKING:
  from ..models.workflow_fields import WorkflowFields





T = TypeVar("T", bound="Workflow")


@_attrs_define
class Workflow:
    """ 
        Attributes:
            id (Union[Unset, UUID]): Workflow identifier.
            agent_id (Union[Unset, UUID]): Identifier of the associated agent.
            name (Union[Unset, str]): Workflow name.
            fields (Union[Unset, WorkflowFields]): Workflow configuration. Example: {'model': 'gpt-4o', 'version':
                '2024-02-01'}.
            prompts (Union[Unset, List[str]]): The prompts for the workflow. They can have variables in them. They will be
                merged with the dynamic data passed when the workflow is executed. Example: ['Your name is {{.name}}, you speak
                {{.language}}', 'Your task is {{.task}}'].
            prompt_variables (Union[Unset, List[str]]): The variables in the prompts. Example: ['name', 'language', 'task'].
     """

    id: Union[Unset, UUID] = UNSET
    agent_id: Union[Unset, UUID] = UNSET
    name: Union[Unset, str] = UNSET
    fields: Union[Unset, 'WorkflowFields'] = UNSET
    prompts: Union[Unset, List[str]] = UNSET
    prompt_variables: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.workflow_fields import WorkflowFields
        id: Union[Unset, str] = UNSET
        if not isinstance(self.id, Unset):
            id = str(self.id)

        agent_id: Union[Unset, str] = UNSET
        if not isinstance(self.agent_id, Unset):
            agent_id = str(self.agent_id)

        name = self.name

        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.fields, Unset):
            fields = self.fields.to_dict()

        prompts: Union[Unset, List[str]] = UNSET
        if not isinstance(self.prompts, Unset):
            prompts = self.prompts



        prompt_variables: Union[Unset, List[str]] = UNSET
        if not isinstance(self.prompt_variables, Unset):
            prompt_variables = self.prompt_variables




        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if id is not UNSET:
            field_dict["id"] = id
        if agent_id is not UNSET:
            field_dict["agent_id"] = agent_id
        if name is not UNSET:
            field_dict["name"] = name
        if fields is not UNSET:
            field_dict["fields"] = fields
        if prompts is not UNSET:
            field_dict["prompts"] = prompts
        if prompt_variables is not UNSET:
            field_dict["prompt_variables"] = prompt_variables

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.workflow_fields import WorkflowFields
        d = src_dict.copy()
        _id = d.pop("id", UNSET)
        id: Union[Unset, UUID]
        if isinstance(_id,  Unset):
            id = UNSET
        else:
            id = UUID(_id)




        _agent_id = d.pop("agent_id", UNSET)
        agent_id: Union[Unset, UUID]
        if isinstance(_agent_id,  Unset):
            agent_id = UNSET
        else:
            agent_id = UUID(_agent_id)




        name = d.pop("name", UNSET)

        _fields = d.pop("fields", UNSET)
        fields: Union[Unset, WorkflowFields]
        if isinstance(_fields,  Unset):
            fields = UNSET
        else:
            fields = WorkflowFields.from_dict(_fields)




        prompts = cast(List[str], d.pop("prompts", UNSET))


        prompt_variables = cast(List[str], d.pop("prompt_variables", UNSET))


        workflow = cls(
            id=id,
            agent_id=agent_id,
            name=name,
            fields=fields,
            prompts=prompts,
            prompt_variables=prompt_variables,
        )


        workflow.additional_properties = d
        return workflow

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import cast, List
from typing import Dict

if TYPE_CHECKING:
  from ..models.tool_call_ids import ToolCallIds





T = TypeVar("T", bound="ChoiceIds")


@_attrs_define
class ChoiceIds:
    """ 
        Attributes:
            choice_id (str):
            message_id (str):
            tool_call_ids (List['ToolCallIds']):
     """

    choice_id: str
    message_id: str
    tool_call_ids: List['ToolCallIds']
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.tool_call_ids import ToolCallIds
        choice_id = self.choice_id

        message_id = self.message_id

        tool_call_ids = []
        for tool_call_ids_item_data in self.tool_call_ids:
            tool_call_ids_item = tool_call_ids_item_data.to_dict()
            tool_call_ids.append(tool_call_ids_item)




        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "choice_id": choice_id,
            "message_id": message_id,
            "tool_call_ids": tool_call_ids,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.tool_call_ids import ToolCallIds
        d = src_dict.copy()
        choice_id = d.pop("choice_id")

        message_id = d.pop("message_id")

        tool_call_ids = []
        _tool_call_ids = d.pop("tool_call_ids")
        for tool_call_ids_item_data in (_tool_call_ids):
            tool_call_ids_item = ToolCallIds.from_dict(tool_call_ids_item_data)



            tool_call_ids.append(tool_call_ids_item)


        choice_ids = cls(
            choice_id=choice_id,
            message_id=message_id,
            tool_call_ids=tool_call_ids,
        )


        choice_ids.additional_properties = d
        return choice_ids

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.asteroid_choice_finish_reason_type_1 import AsteroidChoiceFinishReasonType1
from typing import cast
from typing import cast, Union
from typing import Dict

if TYPE_CHECKING:
  from ..models.asteroid_message import AsteroidMessage





T = TypeVar("T", bound="AsteroidChoice")


@_attrs_define
class AsteroidChoice:
    """ 
        Attributes:
            asteroid_id (str):
            index (int):
            message (AsteroidMessage):
            finish_reason (Union[AsteroidChoiceFinishReasonType1, None]):
     """

    asteroid_id: str
    index: int
    message: 'AsteroidMessage'
    finish_reason: Union[AsteroidChoiceFinishReasonType1, None]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.asteroid_message import AsteroidMessage
        asteroid_id = self.asteroid_id

        index = self.index

        message = self.message.to_dict()

        finish_reason: Union[None, str]
        if isinstance(self.finish_reason, AsteroidChoiceFinishReasonType1):
            finish_reason = self.finish_reason.value
        else:
            finish_reason = self.finish_reason


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "asteroid_id": asteroid_id,
            "index": index,
            "message": message,
            "finish_reason": finish_reason,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.asteroid_message import AsteroidMessage
        d = src_dict.copy()
        asteroid_id = d.pop("asteroid_id")

        index = d.pop("index")

        message = AsteroidMessage.from_dict(d.pop("message"))




        def _parse_finish_reason(data: object) -> Union[AsteroidChoiceFinishReasonType1, None]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                finish_reason_type_1 = AsteroidChoiceFinishReasonType1(data)



                return finish_reason_type_1
            except: # noqa: E722
                pass
            return cast(Union[AsteroidChoiceFinishReasonType1, None], data)

        finish_reason = _parse_finish_reason(d.pop("finish_reason"))


        asteroid_choice = cls(
            asteroid_id=asteroid_id,
            index=index,
            message=message,
            finish_reason=finish_reason,
        )


        asteroid_choice.additional_properties = d
        return asteroid_choice

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import Union






T = TypeVar("T", bound="ToolCallIds")


@_attrs_define
class ToolCallIds:
    """ 
        Attributes:
            tool_call_id (Union[Unset, str]):
            tool_id (Union[Unset, str]):
     """

    tool_call_id: Union[Unset, str] = UNSET
    tool_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        tool_call_id = self.tool_call_id

        tool_id = self.tool_id


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if tool_call_id is not UNSET:
            field_dict["tool_call_id"] = tool_call_id
        if tool_id is not UNSET:
            field_dict["tool_id"] = tool_id

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        tool_call_id = d.pop("tool_call_id", UNSET)

        tool_id = d.pop("tool_id", UNSET)

        tool_call_ids = cls(
            tool_call_id=tool_call_id,
            tool_id=tool_id,
        )


        tool_call_ids.additional_properties = d
        return tool_call_ids

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import cast, List
from typing import Dict

if TYPE_CHECKING:
  from ..models.supervisor_chain import SupervisorChain
  from ..models.chain_execution import ChainExecution
  from ..models.supervision_request_state import SupervisionRequestState





T = TypeVar("T", bound="ChainExecutionState")


@_attrs_define
class ChainExecutionState:
    """ 
        Attributes:
            chain (SupervisorChain):
            chain_execution (ChainExecution):
            supervision_requests (List['SupervisionRequestState']):
     """

    chain: 'SupervisorChain'
    chain_execution: 'ChainExecution'
    supervision_requests: List['SupervisionRequestState']
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.supervisor_chain import SupervisorChain
        from ..models.chain_execution import ChainExecution
        from ..models.supervision_request_state import SupervisionRequestState
        chain = self.chain.to_dict()

        chain_execution = self.chain_execution.to_dict()

        supervision_requests = []
        for supervision_requests_item_data in self.supervision_requests:
            supervision_requests_item = supervision_requests_item_data.to_dict()
            supervision_requests.append(supervision_requests_item)




        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "chain": chain,
            "chain_execution": chain_execution,
            "supervision_requests": supervision_requests,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.supervisor_chain import SupervisorChain
        from ..models.chain_execution import ChainExecution
        from ..models.supervision_request_state import SupervisionRequestState
        d = src_dict.copy()
        chain = SupervisorChain.from_dict(d.pop("chain"))




        chain_execution = ChainExecution.from_dict(d.pop("chain_execution"))




        supervision_requests = []
        _supervision_requests = d.pop("supervision_requests")
        for supervision_requests_item_data in (_supervision_requests):
            supervision_requests_item = SupervisionRequestState.from_dict(supervision_requests_item_data)



            supervision_requests.append(supervision_requests_item)


        chain_execution_state = cls(
            chain=chain,
            chain_execution=chain_execution,
            supervision_requests=supervision_requests,
        )


        chain_execution_state.additional_properties = d
        return chain_execution_state

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset







T = TypeVar("T", bound="SupervisorStatsDecisions")


@_attrs_define
class SupervisorStatsDecisions:
    """ 
        Attributes:
            approve (int):
            reject (int):
            terminate (int):
            modify (int):
            escalate (int):
     """

    approve: int
    reject: int
    terminate: int
    modify: int
    escalate: int
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        approve = self.approve

        reject = self.reject

        terminate = self.terminate

        modify = self.modify

        escalate = self.escalate


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "approve": approve,
            "reject": reject,
            "terminate": terminate,
            "modify": modify,
            "escalate": escalate,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        approve = d.pop("approve")

        reject = d.pop("reject")

        terminate = d.pop("terminate")

        modify = d.pop("modify")

        escalate = d.pop("escalate")

        supervisor_stats_decisions = cls(
            approve=approve,
            reject=reject,
            terminate=terminate,
            modify=modify,
            escalate=escalate,
        )


        supervisor_stats_decisions.additional_properties = d
        return supervisor_stats_decisions

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import cast, List
from typing import Dict
from uuid import UUID

if TYPE_CHECKING:
  from ..models.supervision_request import SupervisionRequest
  from ..models.chain_execution_state import ChainExecutionState
  from ..models.asteroid_tool_call import AsteroidToolCall
  from ..models.asteroid_message import AsteroidMessage





T = TypeVar("T", bound="ReviewPayload")


@_attrs_define
class ReviewPayload:
    """ Contains all the information needed for a human reviewer to make a supervision decision

        Attributes:
            supervision_request (SupervisionRequest):
            chain_state (ChainExecutionState):
            toolcall (AsteroidToolCall):
            run_id (UUID): The ID of the run this review is for
            messages (List['AsteroidMessage']): The messages in the run
     """

    supervision_request: 'SupervisionRequest'
    chain_state: 'ChainExecutionState'
    toolcall: 'AsteroidToolCall'
    run_id: UUID
    messages: List['AsteroidMessage']
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.supervision_request import SupervisionRequest
        from ..models.chain_execution_state import ChainExecutionState
        from ..models.asteroid_tool_call import AsteroidToolCall
        from ..models.asteroid_message import AsteroidMessage
        supervision_request = self.supervision_request.to_dict()

        chain_state = self.chain_state.to_dict()

        toolcall = self.toolcall.to_dict()

        run_id = str(self.run_id)

        messages = []
        for messages_item_data in self.messages:
            messages_item = messages_item_data.to_dict()
            messages.append(messages_item)




        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "supervision_request": supervision_request,
            "chain_state": chain_state,
            "toolcall": toolcall,
            "run_id": run_id,
            "messages": messages,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.supervision_request import SupervisionRequest
        from ..models.chain_execution_state import ChainExecutionState
        from ..models.asteroid_tool_call import AsteroidToolCall
        from ..models.asteroid_message import AsteroidMessage
        d = src_dict.copy()
        supervision_request = SupervisionRequest.from_dict(d.pop("supervision_request"))




        chain_state = ChainExecutionState.from_dict(d.pop("chain_state"))




        toolcall = AsteroidToolCall.from_dict(d.pop("toolcall"))




        run_id = UUID(d.pop("run_id"))




        messages = []
        _messages = d.pop("messages")
        for messages_item_data in (_messages):
            messages_item = AsteroidMessage.from_dict(messages_item_data)



            messages.append(messages_item)


        review_payload = cls(
            supervision_request=supervision_request,
            chain_state=chain_state,
            toolcall=toolcall,
            run_id=run_id,
            messages=messages,
        )


        review_payload.additional_properties = d
        return review_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import cast, List
from typing import Dict
from uuid import UUID

if TYPE_CHECKING:
  from ..models.choice_ids import ChoiceIds





T = TypeVar("T", bound="ChatIds")


@_attrs_define
class ChatIds:
    """ 
        Attributes:
            chat_id (UUID):
            choice_ids (List['ChoiceIds']):
     """

    chat_id: UUID
    choice_ids: List['ChoiceIds']
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.choice_ids import ChoiceIds
        chat_id = str(self.chat_id)

        choice_ids = []
        for choice_ids_item_data in self.choice_ids:
            choice_ids_item = choice_ids_item_data.to_dict()
            choice_ids.append(choice_ids_item)




        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "chat_id": chat_id,
            "choice_ids": choice_ids,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.choice_ids import ChoiceIds
        d = src_dict.copy()
        chat_id = UUID(d.pop("chat_id"))




        choice_ids = []
        _choice_ids = d.pop("choice_ids")
        for choice_ids_item_data in (_choice_ids):
            choice_ids_item = ChoiceIds.from_dict(choice_ids_item_data)



            choice_ids.append(choice_ids_item)


        chat_ids = cls(
            chat_id=chat_id,
            choice_ids=choice_ids,
        )


        chat_ids.additional_properties = d
        return chat_ids

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
from typing import cast
import datetime






T = TypeVar("T", bound="AgentReportTimePeriod")


@_attrs_define
class AgentReportTimePeriod:
    """ 
        Attributes:
            start_time (datetime.datetime):
            end_time (datetime.datetime):
     """

    start_time: datetime.datetime
    end_time: datetime.datetime
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        start_time = self.start_time.isoformat()

        end_time = self.end_time.isoformat()


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "start_time": start_time,
            "end_time": end_time,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start_time = isoparse(d.pop("start_time"))




        end_time = isoparse(d.pop("end_time"))




        agent_report_time_period = cls(
            start_time=start_time,
            end_time=end_time,
        )


        agent_report_time_period.additional_properties = d
        return agent_report_time_period

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.chat_format import ChatFormat






T = TypeVar("T", bound="AsteroidChat")


@_attrs_define
class AsteroidChat:
    """ The raw b64 encoded JSON of the request and response data sent/received from the LLM.

        Attributes:
            request_data (str):
            response_data (str):
            format_ (ChatFormat):
     """

    request_data: str
    response_data: str
    format_: ChatFormat
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        request_data = self.request_data

        response_data = self.response_data

        format_ = self.format_.value


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "request_data": request_data,
            "response_data": response_data,
            "format": format_,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        request_data = d.pop("request_data")

        response_data = d.pop("response_data")

        format_ = ChatFormat(d.pop("format"))




        asteroid_chat = cls(
            request_data=request_data,
            response_data=response_data,
            format_=format_,
        )


        asteroid_chat.additional_properties = d
        return asteroid_chat

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
